@@ -6,7 +6,7 @@ from collections import OrderedDict
 
 from prettytable import PrettyTable
 
-from .errors import ContainerError
+from .errors import ContainerError, EnforceError
 from .utils import is_valid_identifier_not_keyword
 
 logger = logging.getLogger(__name__)
@@ -39,11 +39,14 @@ class EntryInfo:
         or custom handling by passing a function that takes in one argument,
         the value. This function must do one of: return the value back as-is,
         return the value back converted to a corrected form, or raise a
-        ValueError.
+        EnforceError.
     default : optional
         A default value for the trait to have if the user does not specify.
         Keep in mind that this should be the same type as ``enforce`` if you
         are demanding a certain type.
+    enforce_doc : str, optional
+        A human-readable explanation of the enforce field.  Will be printed
+        if the entered information does not follow the ``enforce`` type
 
     Raises
     ------
@@ -59,14 +62,17 @@ class EntryInfo:
         class MyDevice(Device):
 
             my_field = EntryInfo('My generated field')
-            number   = EntryInfo('Device number', enforce=int, default=0)
+            number   = EntryInfo('Device number', enforce=int, default=0,
+                                 enforce_doc='This must be a number')
     """
 
-    def __init__(self, doc=None, optional=True, enforce=None, default=None):
+    def __init__(self, doc=None, optional=True, enforce=None,
+                 default=None, enforce_doc=None):
         self.key = None  # Set later by parent class
         self.doc = doc
         self.enforce = enforce
         self.optional = optional
+        self.enforce_doc = str(enforce_doc or '')
 
         # Explicitly set default to None b/c this is how we ensure mandatory
         # information was set
@@ -91,11 +97,10 @@ class EntryInfo:
 
         Raises
         ------
-        ValueError
+        EnforceError
             If the value is not the correct type, or does not match the
             pattern.
         """
-
         if not self.enforce or value is None:
             return value
 
@@ -109,28 +114,39 @@ class EntryInfo:
             elif value.lower() in false_values:
                 return False
             else:
-                raise ValueError(f'{value} as a string is not interpretable '
-                                 'as a boolean.')
+                raise EnforceError(f'{value} as a string is not interpretable '
+                                   f'as a boolean.  {self.enforce_doc}')
 
         elif callable(self.enforce):
             # Try and convert to type or custom handling
-            # Otherwise raise ValueError for types, custom handling otherwise
-            return self.enforce(value)
+            # Otherwise raise EnforceError for types, custom handling otherwise
+            try:
+                return self.enforce(value)
+            except ValueError as e:
+                if self.enforce_doc:
+                    raise EnforceError(self.enforce_doc) from e
+                else:
+                    raise EnforceError(e)
 
         elif isinstance(self.enforce, (list, tuple, set)):
-            # Check that value is in list, otherwise raise ValueError
+            # Check that value is in list, otherwise raise EnforceError
             if value not in self.enforce:
-                raise ValueError('{} was not found in the enforce list {}'
-                                 ''.format(self.key, self.enforce))
+                raise EnforceError(f'{value} was not found in the enforce '
+                                   f'list {self.enforce}. {self.enforce_doc}')
             return value
 
         elif isinstance(self.enforce, Pattern):
-            # Try and match regex patttern, otherwise raise ValueError
-            if not self.enforce.match(value):
-                raise ValueError(
-                    f'{self.key}={value!r} did not match the enforced pattern'
-                    f'{self.enforce.pattern}'
-                )
+            # Try and match regex patttern, otherwise raise EnforceError
+            try:
+                if not self.enforce.match(value):
+                    raise EnforceError(
+                        f'{self.key}={value!r} did not match the enforced '
+                        f'pattern: ({self.enforce.pattern}). '
+                        f'{self.enforce_doc}'
+                    )
+            except TypeError as e:
+                raise EnforceError(e)
+
             return value
 
         # Invalid enforcement
@@ -167,7 +183,8 @@ class EntryInfo:
 
     def __copy__(self):
         return EntryInfo(doc=self.doc, optional=self.optional,
-                         enforce=self.enforce, default=self.default)
+                         enforce=self.enforce, default=self.default,
+                         enforce_doc=self.enforce_doc)
 
 
 class _HappiItemBase:
